@@ -7,17 +7,19 @@
 #include <string.h>
 
 #include "checksig.h"
+#include "intl.h"
 #include "rpmlib.h"
 #include "rpmlead.h"
 #include "signature.h"
 #include "messages.h"
+#include "lib/misc.h"
 
 int doReSign(int add, char *passPhrase, char **argv)
 {
     int fd, ofd, count;
     struct rpmlead lead;
     unsigned short sigtype;
-    char *rpm, sigtarget[1024];
+    char *rpm, *sigtarget;
     char tmprpm[1024];
     unsigned char buffer[8192];
     Header sig;
@@ -26,23 +28,23 @@ int doReSign(int add, char *passPhrase, char **argv)
 	rpm = *argv++;
 	printf("%s:\n", rpm);
 	if ((fd = open(rpm, O_RDONLY, 0644)) < 0) {
-	    fprintf(stderr, "%s: Open failed\n", rpm);
+	    fprintf(stderr, _("%s: Open failed\n"), rpm);
 	    exit(1);
 	}
 	if (readLead(fd, &lead)) {
-	    fprintf(stderr, "%s: readLead failed\n", rpm);
+	    fprintf(stderr, _("%s: readLead failed\n"), rpm);
 	    exit(1);
 	}
 	if (lead.major == 1) {
-	    fprintf(stderr, "%s: Can't sign v1.0 RPM\n", rpm);
+	    fprintf(stderr, _("%s: Can't sign v1.0 RPM\n"), rpm);
 	    exit(1);
 	}
 	if (lead.major == 2) {
-	    fprintf(stderr, "%s: Can't re-sign v2.0 RPM\n", rpm);
+	    fprintf(stderr, _("%s: Can't re-sign v2.0 RPM\n"), rpm);
 	    exit(1);
 	}
 	if (rpmReadSignature(fd, &sig, lead.signature_type)) {
-	    fprintf(stderr, "%s: rpmReadSignature failed\n", rpm);
+	    fprintf(stderr, _("%s: rpmReadSignature failed\n"), rpm);
 	    exit(1);
 	}
 	if (add != ADD_SIGNATURE) {
@@ -50,19 +52,22 @@ int doReSign(int add, char *passPhrase, char **argv)
 	}
 
 	/* Write the rest to a temp file */
-	strcpy(sigtarget, tempnam(rpmGetVar(RPMVAR_TMPPATH), "rpmsigtarget"));
-	ofd = open(sigtarget, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+	if (makeTempFile(NULL, &sigtarget, &ofd))
+	    exit(1);
+
 	while ((count = read(fd, buffer, sizeof(buffer))) > 0) {
 	    if (count == -1) {
-		perror("Couldn't read the header/archive");
+		perror(_("Couldn't read the header/archive"));
 		close(ofd);
 		unlink(sigtarget);
+		free(sigtarget);
 		exit(1);
 	    }
 	    if (write(ofd, buffer, count) < 0) {
-		perror("Couldn't write header/archive to temp file");
+		perror(_("Couldn't write header/archive to temp file"));
 		close(ofd);
 		unlink(sigtarget);
+		free(sigtarget);
 		exit(1);
 	    }
 	}
@@ -78,6 +83,7 @@ int doReSign(int add, char *passPhrase, char **argv)
 	    close(ofd);
 	    unlink(sigtarget);
 	    unlink(tmprpm);
+	    free(sigtarget);
 	    exit(1);
 	}
 
@@ -96,6 +102,7 @@ int doReSign(int add, char *passPhrase, char **argv)
 	    close(ofd);
 	    unlink(sigtarget);
 	    unlink(tmprpm);
+	    free(sigtarget);
 	    rpmFreeSignature(sig);
 	    exit(1);
 	}
@@ -105,25 +112,28 @@ int doReSign(int add, char *passPhrase, char **argv)
 	fd = open(sigtarget, O_RDONLY);
 	while ((count = read(fd, buffer, sizeof(buffer))) > 0) {
 	    if (count == -1) {
-		perror("Couldn't read sigtarget");
+		perror(_("Couldn't read sigtarget"));
 		close(ofd);
 		close(fd);
 		unlink(sigtarget);
 		unlink(tmprpm);
+		free(sigtarget);
 		exit(1);
 	    }
 	    if (write(ofd, buffer, count) < 0) {
-		perror("Couldn't write package");
+		perror(_("Couldn't write package"));
 		close(ofd);
 		close(fd);
 		unlink(sigtarget);
 		unlink(tmprpm);
+		free(sigtarget);
 		exit(1);
 	    }
 	}
 	close(fd);
 	close(ofd);
 	unlink(sigtarget);
+	free(sigtarget);
 
 	/* Move it in to place */
 	unlink(rpm);
@@ -138,7 +148,7 @@ int doCheckSig(int flags, char **argv)
     int fd, ofd, res, res2, res3, missingKeys;
     struct rpmlead lead;
     char *rpm;
-    char result[1024], sigtarget[1024];
+    char result[1024], * sigtarget;
     unsigned char buffer[8192];
     Header sig;
     HeaderIterator sigIter;
@@ -149,45 +159,47 @@ int doCheckSig(int flags, char **argv)
     while (*argv) {
 	rpm = *argv++;
 	if ((fd = open(rpm, O_RDONLY, 0644)) < 0) {
-	    fprintf(stderr, "%s: Open failed\n", rpm);
+	    fprintf(stderr, _("%s: Open failed\n"), rpm);
 	    res++;
 	    continue;
 	}
 	if (readLead(fd, &lead)) {
-	    fprintf(stderr, "%s: readLead failed\n", rpm);
+	    fprintf(stderr, _("%s: readLead failed\n"), rpm);
 	    res++;
 	    continue;
 	}
 	if (lead.major == 1) {
-	    fprintf(stderr, "%s: No signature available (v1.0 RPM)\n", rpm);
+	    fprintf(stderr, _("%s: No signature available (v1.0 RPM)\n"), rpm);
 	    res++;
 	    continue;
 	}
 	if (rpmReadSignature(fd, &sig, lead.signature_type)) {
-	    fprintf(stderr, "%s: rpmReadSignature failed\n", rpm);
+	    fprintf(stderr, _("%s: rpmReadSignature failed\n"), rpm);
 	    res++;
 	    continue;
 	}
 	if (! sig) {
-	    fprintf(stderr, "%s: No signature available\n", rpm);
+	    fprintf(stderr, _("%s: No signature available\n"), rpm);
 	    res++;
 	    continue;
 	}
 	/* Write the rest to a temp file */
-	strcpy(sigtarget, tempnam(rpmGetVar(RPMVAR_TMPPATH), "rpmsigtarget"));
-	ofd = open(sigtarget, O_WRONLY|O_CREAT|O_TRUNC, 0644);
+	if (makeTempFile(NULL, &sigtarget, &ofd))
+	    exit(1);
 	while ((count = read(fd, buffer, sizeof(buffer))) > 0) {
 	    if (count == -1) {
-		perror("Couldn't read the header/archive");
+		perror(_("Couldn't read the header/archive"));
 		close(ofd);
 		unlink(sigtarget);
+		free(sigtarget);
 		exit(1);
 	    }
 	    if (write(ofd, buffer, count) < 0) {
-		fprintf(stderr, "Unable to write %s", sigtarget);
+		fprintf(stderr, _("Unable to write %s"), sigtarget);
 		perror("");
 		close(ofd);
 		unlink(sigtarget);
+		free(sigtarget);
 		exit(1);
 	    }
 	}
@@ -206,7 +218,8 @@ int doCheckSig(int flags, char **argv)
 	    if ((tag == RPMSIGTAG_PGP) && !(flags & CHECKSIG_PGP)) 
 		continue;
 	    else if ((tag == RPMSIGTAG_MD5 || 
-		      tag == RPMSIGTAG_LITTLEENDIANMD5) 
+		      tag == RPMSIGTAG_LEMD5_2 ||
+		      tag == RPMSIGTAG_LEMD5_1) 
 		      && !(flags & CHECKSIG_MD5)) 
 		continue;
 
@@ -222,7 +235,8 @@ int doCheckSig(int flags, char **argv)
 			res2 = 1;
 			break;
 		      case RPMSIGTAG_MD5:
-		      case RPMSIGTAG_LITTLEENDIANMD5:
+		      case RPMSIGTAG_LEMD5_1:
+		      case RPMSIGTAG_LEMD5_2:
 			strcat(buffer, "MD5 ");
 			res2 = 1;
 			break;
@@ -237,7 +251,7 @@ int doCheckSig(int flags, char **argv)
 			}
 			break;
 		      default:
-			strcat(buffer, "!!! ");
+			strcat(buffer, "?UnknownSignatureType? ");
 			res2 = 1;
 		    }
 		}
@@ -250,7 +264,8 @@ int doCheckSig(int flags, char **argv)
 			strcat(buffer, "size ");
 			break;
 		      case RPMSIGTAG_MD5:
-		      case RPMSIGTAG_LITTLEENDIANMD5:
+		      case RPMSIGTAG_LEMD5_1:
+		      case RPMSIGTAG_LEMD5_2:
 			strcat(buffer, "md5 ");
 			break;
 		      case RPMSIGTAG_PGP:
@@ -265,20 +280,21 @@ int doCheckSig(int flags, char **argv)
 	headerFreeIterator(sigIter);
 	res += res2;
 	unlink(sigtarget);
+	free(sigtarget);
 
 	if (res2) {
 	    if (rpmIsVerbose()) {
 		fprintf(stderr, "%s", buffer);
 	    } else {
 		fprintf(stderr, "%sNOT OK%s\n", buffer,
-			missingKeys ? " (MISSING KEYS)" : "");
+			missingKeys ? _(" (MISSING KEYS)") : "");
 	    }
 	} else {
 	    if (rpmIsVerbose()) {
 		printf("%s", buffer);
 	    } else {
 		printf("%sOK%s\n", buffer,
-		       missingKeys ? " (MISSING KEYS)" : "");
+		       missingKeys ? _(" (MISSING KEYS)") : "");
 	    }
 	}
     }
