@@ -1,9 +1,11 @@
 #if HAVE_CONFIG_H
 # include "config.h"
+# include "miscfn.h"
 #else
 #define HAVE_MACHINE_TYPES_H 1
 #define HAVE_ALLOCA_H 1
 #define HAVE_NETINET_IN_SYSTM_H 1
+#define HAVE_SYS_SOCKET_H 1
 #endif
 
 #if HAVE_MACHINE_TYPES_H
@@ -14,6 +16,10 @@
 # include <alloca.h>
 #endif
 
+#if HAVE_SYS_SOCKET_H
+# include <sys/socket.h>
+#endif
+
 #if HAVE_NETINET_IN_SYSTM_H
 # include <sys/types.h>
 # include <netinet/in_systm.h>
@@ -50,6 +56,10 @@ extern int h_errno;
 # define IPPORT_FTP 21
 #endif
 
+#if defined(USE_ALT_DNS) && USE_ALT_DNS 
+#include "dns.h"
+#endif
+
 #include "ftp.h"
 
 static int ftpCheckResponse(int sock, char ** str);
@@ -152,7 +162,7 @@ int ftpCommand(int sock, char * command, ...) {
     int rc;
 
     va_start(ap, command);
-    len = strlen(command) + 1;
+    len = strlen(command) + 2;
     s = va_arg(ap, char *);
     while (s) {
 	len += strlen(s) + 1;
@@ -173,6 +183,7 @@ int ftpCommand(int sock, char * command, ...) {
     }
     va_end(ap);
 
+    buf[len - 2] = '\r';
     buf[len - 1] = '\n';
     buf[len] = '\0';
      
@@ -186,21 +197,28 @@ int ftpCommand(int sock, char * command, ...) {
     return 0;
 }
 
-static int getHostAddress(const char * host, struct in_addr * address) {
+#if !defined(USE_ALT_DNS) || !USE_ALT_DNS 
+static int mygethostbyname(const char * host, struct in_addr * address) {
     struct hostent * hostinfo;
 
+    hostinfo = gethostbyname(host);
+    if (!hostinfo) return 1;
+
+    memcpy(address, hostinfo->h_addr_list[0], hostinfo->h_length);
+    return 0;
+}
+#endif
+
+static int getHostAddress(const char * host, struct in_addr * address) {
     if (isdigit(host[0])) {
       if (!inet_aton(host, address)) {
 	  return FTPERR_BAD_HOST_ADDR;
       }
     } else {
-      hostinfo = gethostbyname(host);
-      if (!hostinfo) {
+      if (mygethostbyname(host, address)) {
 	  errno = h_errno;
 	  return FTPERR_BAD_HOSTNAME;
       }
-      
-      memcpy(address, hostinfo->h_addr_list[0], hostinfo->h_length);
     }
     
     return 0;
@@ -328,7 +346,7 @@ int ftpGetFileDesc(int sock, char * remotename) {
     char * retrCommand;
     int rc;
 
-    if (write(sock, "PASV\n", 5) != 5) {
+    if (write(sock, "PASV\r\n", 6) != 6) {
         return FTPERR_SERVER_IO_ERROR;
     }
     if ((rc = ftpCheckResponse(sock, &passReply)))
@@ -373,7 +391,7 @@ int ftpGetFileDesc(int sock, char * remotename) {
     }
 
     retrCommand = alloca(strlen(remotename) + 20);
-    sprintf(retrCommand, "RETR %s\n", remotename);
+    sprintf(retrCommand, "RETR %s\r\n", remotename);
     i = strlen(retrCommand);
    
     if (write(sock, retrCommand, i) != i) {
