@@ -10,7 +10,9 @@
 # include <alloca.h>
 #endif
 
+#include "miscfn.h"
 #include "install.h"
+#include "intl.h"
 #include "lib/rpmlib.h"
 #include "messages.h"
 #include "query.h"
@@ -76,7 +78,7 @@ static int installPackages(char * rootdir, char ** packages, char * location,
 
 	fd = open(*filename, O_RDONLY);
 	if (fd < 0) {
-	    fprintf(stderr, "error: cannot open file %s\n", *filename);
+	    fprintf(stderr, _("error: cannot open file %s\n"), *filename);
 	    numFailed++;
 	    *filename = NULL;
 	    continue;
@@ -93,7 +95,7 @@ static int installPackages(char * rootdir, char ** packages, char * location,
 
 	    printFormat = "%-28s";
 	} else if (rpmIsVerbose())
-	    printf("Installing %s\n", *filename);
+	    printf(_("Installing %s\n"), *filename);
 
 	if (db) {
 	    rc = rpmInstallPackage(rootdir, db, fd, location, installFlags, fn, 
@@ -105,17 +107,18 @@ static int installPackages(char * rootdir, char ** packages, char * location,
 		rc = 0;
 	    } else {
 		rc = rpmInstallSourcePackage(rootdir, fd, NULL, fn,
-					     printFormat);
+					     printFormat, NULL);
 	    }
 	} 
 
 	if (rc == 1) {
-	    fprintf(stderr, "error: %s does not appear to be a RPM package\n", 
-			*filename);
+	    fprintf(stderr, 
+		    _("error: %s does not appear to be a RPM package\n"), 
+		    *filename);
 	}
 	    
 	if (rc) {
-	    fprintf(stderr, "error: %s cannot be installed\n", *filename);
+	    fprintf(stderr, _("error: %s cannot be installed\n"), *filename);
 	    numFailed++;
 	}
 
@@ -162,15 +165,17 @@ int doInstall(char * rootdir, char ** argv, char * location, int installFlags,
     for (filename = argv, i = 0; *filename; filename++) {
 	if (urlIsURL(*filename)) {
 	    if (rpmIsVerbose()) {
-		printf("Retrieving %s\n", *filename);
+		printf(_("Retrieving %s\n"), *filename);
 	    }
 	    packages[i] = alloca(strlen(*filename) + 30 + strlen(rootdir));
 	    sprintf(packages[i], "%s/var/tmp/rpm-ftp-%d-%d.tmp", rootdir, 
 		    tmpnum++, (int) getpid());
-	    rpmMessage(RPMMESS_DEBUG, "getting %s as %s\n", *filename, packages[i]);
+	    rpmMessage(RPMMESS_DEBUG, 
+			"getting %s as %s\n", *filename, packages[i]);
 	    fd = urlGetFile(*filename, packages[i]);
 	    if (fd < 0) {
-		fprintf(stderr, "error: skipping %s - transfer failed - %s\n", 
+		fprintf(stderr, 
+			_("error: skipping %s - transfer failed - %s\n"), 
 			*filename, ftpStrerror(fd));
 		numFailed++;
 	    } else {
@@ -188,24 +193,25 @@ int doInstall(char * rootdir, char ** argv, char * location, int installFlags,
     for (filename = packages; *filename; filename++) {
 	fd = open(*filename, O_RDONLY);
 	if (fd < 0) {
-	    fprintf(stderr, "error: cannot open file %s\n", *filename);
+	    fprintf(stderr, _("error: cannot open file %s\n"), *filename);
 	    numFailed++;
 	    *filename = NULL;
 	    continue;
 	}
 
-	rc = rpmReadPackageHeader(fd, &binaryHeaders[numBinaryPackages], &isSource,
-			   NULL, NULL);
+	rc = rpmReadPackageHeader(fd, &binaryHeaders[numBinaryPackages], 
+					&isSource, NULL, NULL);
 
 	close(fd);
 	
 	if (rc == 1) {
-	    fprintf(stderr, "error: %s does not appear to be a RPM package\n", 
+	    fprintf(stderr, 
+			_("error: %s does not appear to be a RPM package\n"), 
 			*filename);
 	}
 	    
 	if (rc) {
-	    fprintf(stderr, "error: %s cannot be installed\n", *filename);
+	    fprintf(stderr, _("error: %s cannot be installed\n"), *filename);
 	    numFailed++;
 	    *filename = NULL;
 	} else if (isSource) {
@@ -225,34 +231,43 @@ int doInstall(char * rootdir, char ** argv, char * location, int installFlags,
     if (numBinaryPackages) {
 	rpmMessage(RPMMESS_DEBUG, "opening database mode: 0%o\n", mode);
 	if (rpmdbOpen(rootdir, &db, mode, 0644)) {
-	    fprintf(stderr, "error: cannot open %s/var/lib/rpm/packages.rpm\n", 
-			rootdir);
+	    fprintf(stderr, _("error: cannot open %s%s/packages.rpm\n"), 
+			rootdir, rpmGetVar(RPMVAR_DBPATH));
 	    exit(1);
 	}
 
-	if (!(interfaceFlags & INSTALL_NODEPS)) {
-	    rpmdep = rpmdepDependencies(db);
-	    for (i = 0; i < numBinaryPackages; i++)
-		if (installFlags & RPMINSTALL_UPGRADE)
-		    rpmdepUpgradePackage(rpmdep, binaryHeaders[i]);
-		else
-		    rpmdepAddPackage(rpmdep, binaryHeaders[i]);
+	rpmdep = rpmdepDependencies(db);
+	for (i = 0; i < numBinaryPackages; i++)
+	    if (installFlags & RPMINSTALL_UPGRADE)
+		rpmdepUpgradePackage(rpmdep, binaryHeaders[i],
+				     packages[i]);
+	    else
+		rpmdepAddPackage(rpmdep, binaryHeaders[i], 
+				    packages[i]);
 
+	if (!(interfaceFlags & INSTALL_NODEPS)) {
 	    if (rpmdepCheck(rpmdep, &conflicts, &numConflicts)) {
 		numFailed = numPackages;
 		stopInstall = 1;
 	    }
 
-	    rpmdepDone(rpmdep);
-
 	    if (!stopInstall && conflicts) {
-		fprintf(stderr, "failed dependencies:\n");
+		fprintf(stderr, _("failed dependencies:\n"));
 		printDepProblems(stderr, conflicts, numConflicts);
 		rpmdepFreeConflicts(conflicts, numConflicts);
 		numFailed = numPackages;
 		stopInstall = 1;
 	    }
 	}
+
+	if (!(interfaceFlags & INSTALL_NOORDER)) {
+	    if (rpmdepOrder(rpmdep, (void ***) &packages)) {
+		numFailed = numPackages;
+		stopInstall = 1;
+	    }
+	}
+
+	rpmdepDone(rpmdep);
     }
     else
 	db = NULL;
@@ -282,7 +297,7 @@ int doUninstall(char * rootdir, char ** argv, int uninstallFlags,
     int mode;
     int rc;
     int count;
-    int numPackages;
+    int numPackages, packageOffsetsAlloced;
     int * packageOffsets;
     char ** arg;
     int numFailed = 0;
@@ -294,9 +309,9 @@ int doUninstall(char * rootdir, char ** argv, int uninstallFlags,
     rpmMessage(RPMMESS_DEBUG, "counting packages to uninstall\n");
     for (arg = argv, numPackages = 0; *arg; arg++, numPackages++)
 	;
-    rpmMessage(RPMMESS_DEBUG, "found %d packages to uninstall\n", numPackages);
 
-    packageOffsets = alloca(sizeof(int *) * numPackages);
+    packageOffsetsAlloced = numPackages;
+    packageOffsets = malloc(sizeof(int *) * packageOffsetsAlloced);
 
     if (uninstallFlags & RPMUNINSTALL_TEST) 
 	mode = O_RDONLY;
@@ -304,29 +319,38 @@ int doUninstall(char * rootdir, char ** argv, int uninstallFlags,
 	mode = O_RDWR | O_EXCL;
 	
     if (rpmdbOpen(rootdir, &db, mode, 0644)) {
-	fprintf(stderr, "cannot open %s/var/lib/rpm/packages.rpm\n", rootdir);
+	fprintf(stderr, _("error: cannot open %s%s/packages.rpm\n"), 
+		rootdir, rpmGetVar(RPMVAR_DBPATH));
 	exit(1);
     }
 
     j = 0;
-    for (arg = argv, numPackages = 0; *arg; arg++, numPackages++) {
-	rc = findPackageByLabel(db, *arg, &matches);
+    numPackages = 0;
+    for (arg = argv; *arg; arg++) {
+	rc = rpmdbFindByLabel(db, *arg, &matches);
 	if (rc == 1) {
-	    fprintf(stderr, "package %s is not installed\n", *arg);
+	    fprintf(stderr, _("package %s is not installed\n"), *arg);
 	    numFailed++;
 	} else if (rc == 2) {
-	    fprintf(stderr, "error searching for package %s\n", *arg);
+	    fprintf(stderr, _("error searching for package %s\n"), *arg);
 	    numFailed++;
 	} else {
 	    count = 0;
 	    for (i = 0; i < matches.count; i++)
 		if (matches.recs[i].recOffset) count++;
 
-	    if (count > 1) {
-		fprintf(stderr, "\"%s\" specifies multiple packages\n", *arg);
+	    if (count > 1 && !(interfaceFlags & UNINSTALL_ALLMATCHES)) {
+		fprintf(stderr, _("\"%s\" specifies multiple packages\n"), 
+			*arg);
 		numFailed++;
 	    }
 	    else { 
+		numPackages += count;
+		if (numPackages > packageOffsetsAlloced) {
+		    packageOffsetsAlloced = numPackages + 5;
+		    packageOffsets = realloc(packageOffsets, 
+				sizeof(int *) * packageOffsetsAlloced);
+		}
 		for (i = 0; i < matches.count; i++) {
 		    if (matches.recs[i].recOffset) {
 			packageOffsets[j++] = matches.recs[i].recOffset;
@@ -337,7 +361,8 @@ int doUninstall(char * rootdir, char ** argv, int uninstallFlags,
 	    dbiFreeIndexRecord(matches);
 	}
     }
-    numPackages = j;
+
+    rpmMessage(RPMMESS_DEBUG, "found %d packages to uninstall\n", numPackages);
 
     if (!(interfaceFlags & UNINSTALL_NODEPS)) {
 	rpmdep = rpmdepDependencies(db);
@@ -352,8 +377,8 @@ int doUninstall(char * rootdir, char ** argv, int uninstallFlags,
 	rpmdepDone(rpmdep);
 
 	if (!stopUninstall && conflicts) {
-	    fprintf(stderr, "removing these packages would break "
-			    "dependencies:\n");
+	    fprintf(stderr, _("removing these packages would break "
+			      "dependencies:\n"));
 	    printDepProblems(stderr, conflicts, numConflicts);
 	    rpmdepFreeConflicts(conflicts, numConflicts);
 	    numFailed += numPackages;
@@ -373,25 +398,30 @@ int doUninstall(char * rootdir, char ** argv, int uninstallFlags,
 
     rpmdbClose(db);
 
+    free(packageOffsets);
+
     return numFailed;
 }
 
-int doSourceInstall(char * rootdir, char * arg, char ** specFile) {
+int doSourceInstall(char * rootdir, char * arg, char ** specFile,
+		    char ** cookie) {
     int fd;
     int rc;
 
     fd = open(arg, O_RDONLY);
     if (fd < 0) {
-	fprintf(stderr, "error: cannot open %s\n", arg);
+	fprintf(stderr, _("error: cannot open %s\n"), arg);
 	return 1;
     }
 
     if (rpmIsVerbose())
 	printf("Installing %s\n", arg);
 
-    rc = rpmInstallSourcePackage(rootdir, fd, specFile, NULL, NULL);
+    rc = rpmInstallSourcePackage(rootdir, fd, specFile, NULL, NULL, cookie);
     if (rc == 1) {
-	fprintf(stderr, "error: %s cannot be installed\n", arg);
+	fprintf(stderr, _("error: %s cannot be installed\n"), arg);
+	if (specFile) free(*specFile);
+	if (cookie) free(*cookie);
     }
 
     close(fd);
@@ -428,10 +458,10 @@ static void printDepProblems(FILE * f, struct rpmDependencyConflict * conflicts,
 	}
 
 	if (conflicts[i].sense == RPMDEP_SENSE_REQUIRES) 
-	    fprintf(f, " is needed by %s-%s-%s\n", conflicts[i].byName, 
+	    fprintf(f, _(" is needed by %s-%s-%s\n"), conflicts[i].byName, 
 		    conflicts[i].byVersion, conflicts[i].byRelease);
 	else
-	    fprintf(f, " conflicts with %s-%s-%s\n", conflicts[i].byName, 
+	    fprintf(f, _(" conflicts with %s-%s-%s\n"), conflicts[i].byName, 
 		    conflicts[i].byVersion, conflicts[i].byRelease);
     }
 }
