@@ -1,9 +1,11 @@
 #include "config.h"
+#include "miscfn.h"
 
 #if HAVE_ALLOCA_H
 # include <alloca.h>
 #endif
 
+#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -14,19 +16,22 @@
 struct availablePackage {
     Header h;
     char ** provides;
+    char ** files;
     char * name, * version, * release;
-    int serial, hasSerial, providesCount;
+    int serial, hasSerial, providesCount, filesCount;
     void * key;
 } ;
 
+enum indexEntryType { IET_NAME, IET_PROVIDES, IET_FILE };
+
 struct availableIndexEntry {
     struct availablePackage * package;
     char * entry;
-    int isProvides;
+    enum indexEntryType type;
 } ;
 
 struct availableIndex {
-    struct availableIndexEntry * index;
+    struct availableIndexEntry * index ;
     int size;
 } ;
 
@@ -61,7 +66,7 @@ static int unsatisfiedDepend(rpmDependencies rpmdep, char * reqName,
 			     char * reqVersion, int reqFlags,
 			     struct availablePackage ** suggestion);
 static int checkDependentPackages(rpmDependencies rpmdep, 
-			    struct problemsSet * psp, char * requires);
+			    struct problemsSet * psp, char * key);
 static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 			Header h, const char * requirement);
 static int dbrecMatchesDepFlags(rpmDependencies rpmdep, int recOffset, 
@@ -74,6 +79,11 @@ static int checkDependentConflicts(rpmDependencies rpmdep,
 			    struct problemsSet * psp, char * package);
 static int checkPackageSet(rpmDependencies rpmdep, struct problemsSet * psp, 
 			    char * package, dbiIndexSet * matches);
+static int addOrderedPack(rpmDependencies rpmdep, 
+			struct availablePackage * package,
+			void ** ordering, int * orderNumPtr, 
+			int * selected, int selectionClass,
+			int satisfyDepends, char ** errorStack);
 
 static void alCreate(struct availableList * al) {
     al->list = malloc(sizeof(*al->list) * 5);
@@ -93,12 +103,14 @@ static void alFreeIndex(struct availableList * al) {
 }
 
 static void alFree(struct availableList * al) {
-     int i;
+    int i;
 
-     for (i = 0; i < al->size; i++) {
+    for (i = 0; i < al->size; i++) {
 	if (al->list[i].provides)
 	    free(al->list[i].provides);
-     }
+	if (al->list[i].files)
+	    free(al->list[i].files);
+    }
 
     if (al->alloced) free(al->list);
     alFreeIndex(al);
@@ -106,7 +118,6 @@ static void alFree(struct availableList * al) {
 
 static void alAddPackage(struct availableList * al, Header h, void * key) {
     struct availablePackage * p;
-    int i, type;
 
     if (al->size == al->alloced) {
 	al->alloced += 5;
@@ -116,17 +127,24 @@ static void alAddPackage(struct availableList * al, Header h, void * key) {
     p = al->list + al->size++;
     p->h = h;
 
-    headerGetEntry(p->h, RPMTAG_NAME, &type, (void **) &p->name, &i);
-    headerGetEntry(p->h, RPMTAG_VERSION, &type, (void **) &p->version, &i);
-    headerGetEntry(p->h, RPMTAG_RELEASE, &type, (void **) &p->release, &i);
-    p->hasSerial = headerGetEntry(h, RPMTAG_SERIAL, &type, (void **) &p->serial, &i);
+    headerGetEntry(p->h, RPMTAG_NAME, NULL, (void **) &p->name, NULL);
+    headerGetEntry(p->h, RPMTAG_VERSION, NULL, (void **) &p->version, NULL);
+    headerGetEntry(p->h, RPMTAG_RELEASE, NULL, (void **) &p->release, NULL);
+    p->hasSerial = headerGetEntry(h, RPMTAG_SERIAL, NULL, (void **) &p->serial, 
+				  NULL);
 
-    if (!headerGetEntry(h, RPMTAG_PROVIDES, &type, (void **) &p->provides,
+    if (!headerGetEntry(h, RPMTAG_PROVIDES, NULL, (void **) &p->provides,
 	&p->providesCount)) {
 	p->providesCount = 0;
 	p->provides = NULL;
     }
 
+    if (!headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &p->files,
+	&p->filesCount)) {
+	p->filesCount = 0;
+	p->files = NULL;
+    }
+
     p->key = key;
 
     alFreeIndex(al);
@@ -142,6 +160,9 @@ static void alMakeIndex(struct availableList * al) {
     for (i = 0; i < al->size; i++) {
 	ai->size += al->list[i].providesCount;
     }
+    for (i = 0; i < al->size; i++) {
+	ai->size += al->list[i].filesCount;
+    }
 
     if (ai->size) {
 	ai->index = malloc(sizeof(*ai->index) * ai->size);
@@ -149,13 +170,20 @@ static void alMakeIndex(struct availableList * al) {
 	for (i = 0; i < al->size; i++) {
 	    ai->index[k].package = al->list + i;
 	    ai->index[k].entry = al->list[i].name;
-	    ai->index[k].isProvides = 0;
+	    ai->index[k].type = IET_NAME;
 	    k++;
 
 	    for (j = 0; j < al->list[i].providesCount; j++) {
 		ai->index[k].package = al->list + i;
 		ai->index[k].entry = al->list[i].provides[j];
-		ai->index[k].isProvides = 1;
+		ai->index[k].type = IET_PROVIDES;
+		k++;
+	    }
+
+	    for (j = 0; j < al->list[i].filesCount; j++) {
+		ai->index[k].package = al->list + i;
+		ai->index[k].entry = al->list[i].files[j];
+		ai->index[k].type = IET_FILE;
 		k++;
 	    }
 	}
@@ -176,7 +204,7 @@ struct availablePackage * alSatisfiesDepend(struct availableList * al,
 		    sizeof(*al->index.index), (void *) indexcmp);
  
     if (!match) return NULL;
-    if (match->isProvides) return match->package;
+    if (match->type != IET_NAME) return match->package;
 
     if (headerMatchesDepFlags(match->package->h, reqVersion, reqFlags))
 	return match->package;
@@ -219,17 +247,18 @@ rpmDependencies rpmdepDependencies(rpmdb db) {
     return rpmdep;
 }
 
-void rpmdepUpgradePackage(rpmDependencies rpmdep, Header h) {
+void rpmdepUpgradePackage(rpmDependencies rpmdep, Header h, void * key) {
     /* this is an install followed by uninstalls */
     dbiIndexSet matches;
     char * name;
-    int count, type, i;
+    int count, i, j;
+    char ** obsoletes;
 
-    alAddPackage(&rpmdep->addedPackages, h, NULL);
+    alAddPackage(&rpmdep->addedPackages, h, key);
 
     if (!rpmdep->db) return;
 
-    headerGetEntry(h, RPMTAG_NAME, &type, (void *) &name, &count);
+    headerGetEntry(h, RPMTAG_NAME, NULL, (void *) &name, &count);
 
     if (!rpmdbFindPackage(rpmdep->db, name, &matches))  {
 	for (i = 0; i < matches.count; i++) {
@@ -238,10 +267,25 @@ void rpmdepUpgradePackage(rpmDependencies rpmdep, Header h) {
 
 	dbiFreeIndexRecord(matches);
     }
+
+    if (headerGetEntry(h, RPMTAG_OBSOLETES, NULL, (void *) &obsoletes, 
+			&count)) {
+	for (j = 0; j < count; j++) {
+	    if (!rpmdbFindPackage(rpmdep->db, obsoletes[j], &matches))  {
+		for (i = 0; i < matches.count; i++) {
+		    rpmdepRemovePackage(rpmdep, matches.recs[i].recOffset);
+		}
+
+		dbiFreeIndexRecord(matches);
+	    }
+	}
+
+	free(obsoletes);
+    }
 }
 
-void rpmdepAddPackage(rpmDependencies rpmdep, Header h) {
-    alAddPackage(&rpmdep->addedPackages, h, NULL);
+void rpmdepAddPackage(rpmDependencies rpmdep, Header h, void * key) {
+    alAddPackage(&rpmdep->addedPackages, h, key);
 }
 
 void rpmdepAvailablePackage(rpmDependencies rpmdep, Header h, void * key) {
@@ -285,8 +329,8 @@ int rpmdepCheck(rpmDependencies rpmdep,
 		struct rpmDependencyConflict ** conflicts, int * numConflicts) {
     struct availablePackage * p;
     int i, j;
-    char ** provides;
-    int providesCount;
+    char ** provides, ** files;
+    int providesCount, fileCount;
     int type;
     char * name;
     Header h;
@@ -333,7 +377,8 @@ int rpmdepCheck(rpmDependencies rpmdep,
     for (i = 0; i < rpmdep->numRemovedPackages; i++) {
 	h = rpmdbGetRecord(rpmdep->db, rpmdep->removedPackages[i]);
 	if (!h) {
-	    rpmError(RPMERR_DBCORRUPT, "cannot read header at %d for dependency "
+	    rpmError(RPMERR_DBCORRUPT, 
+			"cannot read header at %d for dependency "
 		  "check", rpmdep->removedPackages[i]);
 	    free(ps.problems);
 	    return 1;
@@ -347,18 +392,32 @@ int rpmdepCheck(rpmDependencies rpmdep,
 	    return 1;
 	}
 
-	if (!headerGetEntry(h, RPMTAG_PROVIDES, &type, (void **) &provides, 
+	if (headerGetEntry(h, RPMTAG_PROVIDES, NULL, (void **) &provides, 
 		 &providesCount)) {
-	    headerFree(h);
-	    continue;
+	    for (j = 0; j < providesCount; j++) {
+		if (checkDependentPackages(rpmdep, &ps, provides[j])) {
+		    free(provides);
+		    free(ps.problems);
+		    headerFree(h);
+		    return 1;
+		}
+	    }
+
+	    free(provides);
 	}
 
-	for (j = 0; j < providesCount; j++) {
-	    if (checkDependentPackages(rpmdep, &ps, provides[j])) {
-		free(ps.problems);
-		headerFree(h);
-		return 1;
+	if (headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &files, 
+		 &fileCount)) {
+	    for (j = 0; j < fileCount; j++) {
+		if (checkDependentPackages(rpmdep, &ps, files[j])) {
+		    free(files);
+		    free(ps.problems);
+		    headerFree(h);
+		    return 1;
+		}
 	    }
+
+	    free(files);
 	}
 
 	headerFree(h);
@@ -381,44 +440,76 @@ static int unsatisfiedDepend(rpmDependencies rpmdep, char * reqName,
 			     struct availablePackage ** suggestion) {
     dbiIndexSet matches;
     int i;
-
+    char * rcProvidesString;
+    char * start;
+    
     rpmMessage(RPMMESS_DEBUG, "dependencies: looking for %s\n", reqName);
 
     if (suggestion) *suggestion = NULL;
 
+    if (!(reqFlags & RPMSENSE_SENSEMASK) &&
+	(rcProvidesString = rpmGetVar(RPMVAR_PROVIDES))) {
+	i = strlen(reqName);
+	while ((start = strstr(rcProvidesString, reqName))) {
+	    if (isspace(start[i]) || !start[i])
+		return 0;
+	    start += i;
+	}
+    }
+
     if (alSatisfiesDepend(&rpmdep->addedPackages, reqName, reqVersion, 
 			  reqFlags))
 	return 0;
 
     if (rpmdep->db) {
-	if (!reqFlags && !rpmdbFindByProvides(rpmdep->db, reqName, &matches)) {
-	    for (i = 0; i < matches.count; i++) {
-		if (bsearch(&matches.recs[i].recOffset, 
-			    rpmdep->removedPackages, 
-			    rpmdep->numRemovedPackages, sizeof(int *), intcmp)) 
-		    continue;
-		break;
-	    }
-
-	    dbiFreeIndexRecord(matches);
-	    if (i < matches.count) return 0;
-	}
-
-	if (!rpmdbFindPackage(rpmdep->db, reqName, &matches)) {
-	    for (i = 0; i < matches.count; i++) {
-		if (bsearch(&matches.recs[i].recOffset, 
-			    rpmdep->removedPackages, 
-			    rpmdep->numRemovedPackages, sizeof(int *), intcmp)) 
-		    continue;
-
-		if (dbrecMatchesDepFlags(rpmdep, matches.recs[i].recOffset, 
-					 reqVersion, reqFlags)) {
+	if (*reqName == '/') {
+	    /* reqFlags better be 0! */
+	    if (!rpmdbFindByFile(rpmdep->db, reqName, &matches)) {
+		for (i = 0; i < matches.count; i++) {
+		    if (bsearch(&matches.recs[i].recOffset, 
+				rpmdep->removedPackages, 
+				rpmdep->numRemovedPackages, 
+				sizeof(int), intcmp)) 
+			continue;
 		    break;
 		}
+
+		dbiFreeIndexRecord(matches);
+		if (i < matches.count) return 0;
+	    }
+	} else {
+	    if (!reqFlags && !rpmdbFindByProvides(rpmdep->db, reqName, 
+						  &matches)) {
+		for (i = 0; i < matches.count; i++) {
+		    if (bsearch(&matches.recs[i].recOffset, 
+				rpmdep->removedPackages, 
+				rpmdep->numRemovedPackages, 
+				sizeof(int), intcmp)) 
+			continue;
+		    break;
+		}
+
+		dbiFreeIndexRecord(matches);
+		if (i < matches.count) return 0;
 	    }
 
-	    dbiFreeIndexRecord(matches);
-	    if (i < matches.count) return 0;
+	    if (!rpmdbFindPackage(rpmdep->db, reqName, &matches)) {
+		for (i = 0; i < matches.count; i++) {
+		    if (bsearch(&matches.recs[i].recOffset, 
+				rpmdep->removedPackages, 
+				rpmdep->numRemovedPackages, 
+				sizeof(int), intcmp)) 
+			continue;
+
+		    if (dbrecMatchesDepFlags(rpmdep, matches.recs[i].recOffset, 
+					     reqVersion, reqFlags)) {
+			break;
+		    }
+		}
+
+		dbiFreeIndexRecord(matches);
+		if (i < matches.count) return 0;
+	    }
 	}
     }
 
@@ -436,7 +527,7 @@ static int checkPackageSet(rpmDependencies rpmdep, struct problemsSet * psp,
 
     for (i = 0; i < matches->count; i++) {
 	if (bsearch(&matches->recs[i].recOffset, rpmdep->removedPackages, 
-		    rpmdep->numRemovedPackages, sizeof(int *), intcmp)) 
+		    rpmdep->numRemovedPackages, sizeof(int), intcmp)) 
 	    continue;
 
 	h = rpmdbGetRecord(rpmdep->db, matches->recs[i].recOffset);
@@ -458,15 +549,15 @@ static int checkPackageSet(rpmDependencies rpmdep, struct problemsSet * psp,
 }
 
 static int checkDependentPackages(rpmDependencies rpmdep, 
-			    struct problemsSet * psp, char * package) {
+			    struct problemsSet * psp, char * key) {
     dbiIndexSet matches;
     int rc;
 
-    if (rpmdbFindByRequiredBy(rpmdep->db, package, &matches))  {
+    if (rpmdbFindByRequiredBy(rpmdep->db, key, &matches))  {
 	return 0;
     }
 
-    rc = checkPackageSet(rpmdep, psp, package, &matches);
+    rc = checkPackageSet(rpmdep, psp, key, &matches);
     dbiFreeIndexRecord(matches);
 
     return rc;
@@ -506,7 +597,8 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
     } else {
 	headerGetEntry(h, RPMTAG_REQUIREFLAGS, &type, (void **) &requireFlags, 
 		 &requiresCount);
-	headerGetEntry(h, RPMTAG_REQUIREVERSION, &type, (void **) &requiresVersion, 
+	headerGetEntry(h, RPMTAG_REQUIREVERSION, &type, 
+			(void **) &requiresVersion, 
 		 &requiresCount);
     }
 
@@ -514,9 +606,10 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 	     &conflictsCount)) {
 	conflictsCount = 0;
     } else {
-	headerGetEntry(h, RPMTAG_CONFLICTFLAGS, &type, (void **) &conflictsFlags, 
-		 &conflictsCount);
-	headerGetEntry(h, RPMTAG_CONFLICTVERSION, &type,(void **) &conflictsVersion, 
+	headerGetEntry(h, RPMTAG_CONFLICTFLAGS, &type, 
+			(void **) &conflictsFlags, &conflictsCount);
+	headerGetEntry(h, RPMTAG_CONFLICTVERSION, &type,
+			(void **) &conflictsVersion, 
 		 &conflictsCount);
     }
 
@@ -527,8 +620,10 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 			       requireFlags[i], &suggestion);
 	if (rc == 1) {
 	    headerGetEntry(h, RPMTAG_NAME, &type, (void **) &name, &count);
-	    headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version, &count);
-	    headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release, &count);
+	    headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version, 
+				&count);
+	    headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release, 
+				&count);
 
 	    rpmMessage(RPMMESS_DEBUG, "package %s require not satisfied: %s\n",
 		    name, requires[i]);
@@ -567,8 +662,10 @@ static int checkPackageDeps(rpmDependencies rpmdep, struct problemsSet * psp,
 	/* 1 == unsatisfied, 0 == satsisfied */
 	if (rc == 0) {
 	    headerGetEntry(h, RPMTAG_NAME, &type, (void **) &name, &count);
-	    headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version, &count);
-	    headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release, &count);
+	    headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version, 
+				&count);
+	    headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release, 
+				&count);
 
 	    rpmMessage(RPMMESS_DEBUG, "package %s conflicts: %s\n",
 		    name, conflicts[i]);
@@ -612,23 +709,22 @@ static int headerMatchesDepFlags(Header h, char * reqInfo, int reqFlags) {
     char * reqVersion = reqInfo;
     char * reqRelease = NULL;
     int type, count;
-    int_32 serial;
+    int_32 * serial;
     char buf[20];
     int result = 0;
     int sense;
 
     headerGetEntry(h, RPMTAG_NAME, &type, (void *) &name, &count);
 
-    if (!reqFlags) {
+    if (!(reqFlags & RPMSENSE_SENSEMASK)) {
 	return 1;
     }
 
     if (reqFlags & RPMSENSE_SERIAL) {
 	if (!headerGetEntry(h, RPMTAG_SERIAL, &type, (void *) &serial, &count)) {
-	    headerFree(h);
 	    return 0;
 	}
-	sprintf(buf, "%d", serial);
+	sprintf(buf, "%d", *serial);
 	version = buf;
     } else {
 	headerGetEntry(h, RPMTAG_VERSION, &type, (void *) &version, &count);
@@ -679,3 +775,130 @@ static int dbrecMatchesDepFlags(rpmDependencies rpmdep, int recOffset,
 
     return rc;
 }
+
+/* selection status is one of:
+
+	-1:	selected
+	0:	not selected
+	> 0:	selection class
+
+   the current selection pass is included as a separate parameter, and is
+   incremented when satisfying a prerequisite */
+
+static int addOrderedPack(rpmDependencies rpmdep, 
+			struct availablePackage * package,
+			void ** ordering, int * orderNumPtr, 
+			int * selected, int selectionClass,
+			int satisfyDepends, char ** errorStack) {
+    char ** requires, ** requiresVersion;
+    int_32 * requireFlags;
+    int requiresCount;
+    int matchNum;
+    int packageNum = package - rpmdep->addedPackages.list;
+    int i, rc;
+    struct availablePackage * match;
+    char * errorString;
+    char ** stack;
+
+    *errorStack++ = package->name;
+
+    if (selected[packageNum] > 0) {
+	i = 0;
+	stack = errorStack - 1;
+	while (*(--stack)) {
+	    i += strlen(*stack) + 1;
+	}
+
+	errorString = alloca(i + 2);
+	*errorString = '\0';
+	
+	while ((++stack) < errorStack) {
+	    strcat(errorString, *stack);
+	    strcat(errorString, " ");
+	}
+	
+	rpmError(RPMMESS_PREREQLOOP, "loop in prerequisite chain: %s",
+		 errorString);
+
+	return 1;
+    }
+
+    selected[packageNum] = selectionClass;
+
+    if (headerGetEntry(package->h, RPMTAG_REQUIRENAME, NULL, 
+			(void **) &requires, &requiresCount)) {
+	headerGetEntry(package->h, RPMTAG_REQUIREFLAGS, NULL, 
+			(void **) &requireFlags, NULL);
+	headerGetEntry(package->h, RPMTAG_REQUIREVERSION, NULL, 
+			(void **) &requiresVersion, NULL);
+
+	for (i = 0; i < requiresCount; i++) {
+	    if (satisfyDepends || (requireFlags[i] & RPMSENSE_PREREQ)) {
+		match = alSatisfiesDepend(&rpmdep->addedPackages,
+					  requires[i], requiresVersion[i],
+					  requireFlags[i]);
+		/* broken dependencies don't concern us */
+		if (!match) continue;
+		
+		/* let this package satisfy its own predependencies */
+		if (match == package) continue;
+
+		/* the package has already been selected */
+		matchNum = match - rpmdep->addedPackages.list;
+		if (selected[matchNum] == -1 ||
+		    selected[matchNum] == selectionClass)
+		    continue;
+		
+		if (requireFlags[i] & RPMSENSE_PREREQ)
+		    rc = addOrderedPack(rpmdep, match, ordering, orderNumPtr,
+				        selected, selectionClass + 1, 1,
+				        errorStack);
+		else
+		    rc = addOrderedPack(rpmdep, match, ordering, orderNumPtr,
+				        selected, selectionClass, 1,
+				        errorStack);
+	    }
+	}
+    }
+
+    /* whew -- add this package */
+    ordering[(*orderNumPtr)++] = package->key;
+    selected[packageNum] = -1;
+
+    return 0;
+}
+
+int rpmdepOrder(rpmDependencies rpmdep, void *** keysListPtr) {
+    int i;
+    int * selected;
+    void ** order;
+    int orderNum;
+    char ** errorStack;
+
+    alMakeIndex(&rpmdep->addedPackages);
+    alMakeIndex(&rpmdep->availablePackages);
+
+    selected = alloca(sizeof(*selected) * rpmdep->addedPackages.size);
+    memset(selected, 0, sizeof(*selected) * rpmdep->addedPackages.size);
+
+    errorStack = alloca(sizeof(*errorStack) * (rpmdep->addedPackages.size + 1));
+    *errorStack++ = NULL;
+
+    order = malloc(sizeof(*order) * (rpmdep->addedPackages.size + 1));
+    orderNum = 0;
+
+    for (i = 0; i < rpmdep->addedPackages.size; i++) {
+	if (!selected[i]) {
+	    if (addOrderedPack(rpmdep, rpmdep->addedPackages.list + i,
+			       order, &orderNum, selected, 1, 0, errorStack)) {
+		free(order);
+		return 1;
+	    }
+	}
+    }
+
+    order[orderNum] = NULL;
+    *keysListPtr = order;
+
+    return 0;
+}
