@@ -1,17 +1,30 @@
+#include "config.h"
+#include "miscfn.h"
+
+#if HAVE_ALLOCA_H
+# include <alloca.h>
+#endif
+
 #include <errno.h>
 #include <ctype.h>
+#include <fcntl.h>
 #include <grp.h>
 #include <pwd.h>
 #include <stdlib.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 
+#include "intl.h"
 #include "misc.h"
 #include "rpmlib.h"
 #include "messages.h"
 
+char * RPMVERSION = VERSION;
+
 char ** splitString(char * str, int length, char sep) {
     char * s, * source, * dest;
     char ** list;
@@ -179,17 +192,20 @@ int dosetenv(const char *name, const char *value, int overwrite) {
    is looked up via getpw() and getgr() functions.  If this performs
    too poorly I'll have to implement it properly :-( */
 
-uid_t unameToUid(char * thisUname) {
+int unameToUid(char * thisUname, uid_t * uid) {
     static char * lastUname = NULL;
     static int lastUnameLen = 0;
     static int lastUnameAlloced;
     static uid_t lastUid;
     struct passwd * pwent;
     int thisUnameLen;
-   
+
     if (!thisUname) {
 	lastUnameLen = 0;
 	return -1;
+    } else if (!strcmp(thisUname, "root")) {
+	*uid = 0;
+	return 0;
     }
 
     thisUnameLen = strlen(thisUname);
@@ -203,17 +219,20 @@ uid_t unameToUid(char * thisUname) {
 
 	pwent = getpwnam(thisUname);
 	if (!pwent) {
-	    lastUnameLen = 0;
-	    lastUid = -1;
-	} else {
-	    lastUid = pwent->pw_uid;
+	    endpwent();
+	    pwent = getpwnam(thisUname);
+	    if (!pwent) return -1;
 	}
+
+	lastUid = pwent->pw_uid;
     }
 
-    return lastUid;
+    *uid = lastUid;
+
+    return 0;
 }
 
-uid_t gnameToGid(char * thisGname) {
+int gnameToGid(char * thisGname, gid_t * gid) {
     static char * lastGname = NULL;
     static int lastGnameLen = 0;
     static int lastGnameAlloced;
@@ -224,6 +243,9 @@ uid_t gnameToGid(char * thisGname) {
     if (!thisGname) {
 	lastGnameLen = 0;
 	return -1;
+    } else if (!strcmp(thisGname, "root")) {
+	*gid = 0;
+	return 0;
     }
    
     thisGnameLen = strlen(thisGname);
@@ -237,14 +259,16 @@ uid_t gnameToGid(char * thisGname) {
 
 	grent = getgrnam(thisGname);
 	if (!grent) {
-	    lastGnameLen = 0;
-	    lastGid = -1;
-	} else {
-	    lastGid = grent->gr_gid;
+	    endgrent();
+	    grent = getgrnam(thisGname);
+	    if (!grent) return -1;
 	}
+	lastGid = grent->gr_gid;
     }
 
-    return lastGid;
+    *gid = lastGid;
+
+    return 0;
 }
 
 char * uidToUname(uid_t uid) {
@@ -257,6 +281,8 @@ char * uidToUname(uid_t uid) {
     if (uid == (uid_t) -1) {
 	lastUid = -1;
 	return NULL;
+    } else if (!uid) {
+	return "root";
     } else if (uid == lastUid) {
 	return lastUname;
     } else {
@@ -282,9 +308,11 @@ char * gidToGname(gid_t gid) {
     struct group * grent;
     int len;
 
-    if (gid == (uid_t) -1) {
+    if (gid == (gid_t) -1) {
 	lastGid = -1;
 	return NULL;
+    } else if (!gid) {
+	return "root";
     } else if (gid == lastGid) {
 	return lastGname;
     } else {
@@ -302,3 +330,38 @@ char * gidToGname(gid_t gid) {
 	return lastGname;
     }
 }
+
+int makeTempFile(char * prefix, char ** fnptr, int * fdptr) {
+    char * fn;
+    int fd;
+    int ran;
+    char * tmpdir = rpmGetVar(RPMVAR_TMPPATH);
+    struct stat sb;
+
+    if (!prefix) prefix = "";
+
+    fn = malloc(strlen(prefix) + 25 + strlen(tmpdir));
+
+    srand(time(NULL));
+    ran = rand() % 100000;
+    do {
+	sprintf(fn, "%s%s/rpm-tmp.%d", prefix, tmpdir, ran++);
+    } while (!access(fn, X_OK));
+    
+    fd = open(fn, O_CREAT | O_RDWR | O_EXCL, 0700);
+
+    if (fd < 0) {
+	rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), fn);
+	return 1;
+    }
+
+    if (!stat(fn, &sb) && S_ISLNK(sb.st_mode)) {
+	rpmError(RPMERR_SCRIPT, _("error creating temporary file %s"), fn);
+	return 1;
+    }
+
+    if (fnptr) *fnptr = fn;
+    *fdptr = fd;
+
+    return 0;
+}
