@@ -1,27 +1,32 @@
-#include <alloca.h>
+#include "config.h"
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/stat.h>	/* for mkdir(2) !?! */
 #include <unistd.h>
 
+#if HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 #include "messages.h"
 #include "rpmdb.h"
-#include "rpmerr.h"
 #include "rpmlib.h"
 
 int rpmdbRebuild(char * rootdir) {
     rpmdb olddb, newdb;
     char * dbpath, * newdbpath;
-    unsigned int recnum; 
+    int recnum; 
     Header h;
     int failed = 0;
 
-    message(MESS_DEBUG, "rebuilding database in rootdir %s\n", rootdir);
+    rpmMessage(RPMMESS_DEBUG, "rebuilding database in rootdir %s\n", rootdir);
 
-    dbpath = getVar(RPMVAR_DBPATH);
+    dbpath = rpmGetVar(RPMVAR_DBPATH);
     if (!dbpath) {
-	message(MESS_DEBUG, "no dbpath has been set");
+	rpmMessage(RPMMESS_DEBUG, "no dbpath has been set");
 	return 1;
     }
 
@@ -29,40 +34,51 @@ int rpmdbRebuild(char * rootdir) {
     sprintf(newdbpath, "%s/%s/rebuilddb.%d", rootdir, dbpath, (int) getpid());
 
     if (!access(newdbpath, F_OK)) {
-	error(RPMERR_MKDIR, "temporary database %s already exists",
+	rpmError(RPMERR_MKDIR, "temporary database %s already exists",
 	      newdbpath);
     }
 
-    message(MESS_DEBUG, "creating directory: %s\n", newdbpath);
+    rpmMessage(RPMMESS_DEBUG, "creating directory: %s\n", newdbpath);
     if (mkdir(newdbpath, 0755)) {
-	error(RPMERR_MKDIR, "error creating directory %s: %s",
+	rpmError(RPMERR_MKDIR, "error creating directory %s: %s",
 	      newdbpath, strerror(errno));
     }
 
     sprintf(newdbpath, "%s/rebuilddb.%d", dbpath, (int) getpid());
 
-    message(MESS_DEBUG, "opening old database\n");
+    rpmMessage(RPMMESS_DEBUG, "opening old database\n");
     if (openDatabase(rootdir, dbpath, &olddb, O_RDONLY, 0644, 0)) {
 	return 1;
     }
 
-    message(MESS_DEBUG, "opening new database\n");
+    rpmMessage(RPMMESS_DEBUG, "opening new database\n");
     if (openDatabase(rootdir, newdbpath, &newdb, O_RDWR | O_CREAT, 0644, 0)) {
 	return 1;
     }
 
     recnum = rpmdbFirstRecNum(olddb);
-    while (recnum) {
+    while (recnum > 0) {
 	if (!(h = rpmdbGetRecord(olddb, recnum))) {
-	    error(RPMERR_INTERNAL, "cannot read database record at %d", recnum);
+	    rpmError(RPMERR_INTERNAL, "cannot read database record at %d", recnum);
 	    failed = 1;
 	    break;
 	}
-	if (rpmdbAdd(newdb, h)) {
-	    error(RPMERR_INTERNAL, "cannot add record originally at %d", 
-		  recnum);
-	    failed = 1;
-	    break;
+
+	/* let's sanity check this record a bit, otherwise just skip it */
+	if (headerIsEntry(h, RPMTAG_NAME) &&
+	    headerIsEntry(h, RPMTAG_VERSION) &&
+	    headerIsEntry(h, RPMTAG_RELEASE) &&
+	    headerIsEntry(h, RPMTAG_RELEASE) &&
+	    headerIsEntry(h, RPMTAG_BUILDTIME)) {
+	    if (rpmdbAdd(newdb, h)) {
+		rpmError(RPMERR_INTERNAL, "cannot add record originally at %d", 
+		      recnum);
+		failed = 1;
+		break;
+	    }
+	} else {
+	    rpmError(RPMERR_INTERNAL, "record number %d in database is bad "
+			"-- skipping it", recnum);
 	}
 	recnum = rpmdbNextRecNum(olddb, recnum);
     }
@@ -71,16 +87,16 @@ int rpmdbRebuild(char * rootdir) {
     rpmdbClose(newdb);
 
     if (failed) {
-	message(MESS_NORMAL, "failed to rebuild database; original database "
+	rpmMessage(RPMMESS_NORMAL, "failed to rebuild database; original database "
 		"remains in place\n");
 
 	rpmdbRemoveDatabase(rootdir, newdbpath);
 	return 1;
     } else {
 	if (rpmdbMoveDatabase(rootdir, newdbpath, dbpath)) {
-	    message(MESS_ERROR, "failed to replace old database with new "
+	    rpmMessage(RPMMESS_ERROR, "failed to replace old database with new "
 			"database!\n");
-	    message(MESS_ERROR, "replaces files in %s with files from %s "
+	    rpmMessage(RPMMESS_ERROR, "replaces files in %s with files from %s "
 			"to recover", dbpath, newdbpath);
 	    return 1;
 	}
