@@ -15,6 +15,7 @@
 #include "dbindex.h"
 #include "falloc.h"
 #include "header.h"
+#include "intl.h"
 #include "misc.h"
 #include "rpmdb.h"
 #include "rpmlib.h"
@@ -31,7 +32,7 @@
    one. Version numbers still need verification, but it gets us in the
    right area w/o a linear search through the database. */
 
-struct rpmdb {
+struct rpmdb_s {
     faFile pkgs;
     dbiIndex * nameIndex, * fileIndex, * groupIndex, * providesIndex;
     dbiIndex * requiredbyIndex, * conflictsIndex;
@@ -74,7 +75,7 @@ int rpmdbInit (char * prefix, int perms) {
 int openDatabase(char * prefix, char * dbpath, rpmdb *rpmdbp, int mode, 
 		 int perms, int justcheck) {
     char * filename;
-    struct rpmdb db;
+    struct rpmdb_s db;
     int i;
     struct flock lockinfo;
 
@@ -104,7 +105,7 @@ int openDatabase(char * prefix, char * dbpath, rpmdb *rpmdbp, int mode,
     if (!justcheck || !exists(filename)) {
 	db.pkgs = faOpen(filename, mode, 0644);
 	if (!db.pkgs) {
-	    rpmError(RPMERR_DBOPEN, "failed to open %s\n", filename);
+	    rpmError(RPMERR_DBOPEN, _("failed to open %s\n"), filename);
 	    return 1;
 	}
 
@@ -117,14 +118,15 @@ int openDatabase(char * prefix, char * dbpath, rpmdb *rpmdbp, int mode,
 	if (mode & O_RDWR) {
 	    lockinfo.l_type = F_WRLCK;
 	    if (fcntl(db.pkgs->fd, F_SETLK, (void *) &lockinfo)) {
-		rpmError(RPMERR_FLOCK, "cannot get %s lock on database", 
-			"exclusive");
+		rpmError(RPMERR_FLOCK, _("cannot get %s lock on database"), 
+			 _("exclusive"));
 		return 1;
 	    } 
 	} else {
 	    lockinfo.l_type = F_RDLCK;
 	    if (fcntl(db.pkgs->fd, F_SETLK, (void *) &lockinfo)) {
-		rpmError(RPMERR_FLOCK, "cannot get %s lock on database", "shared");
+		rpmError(RPMERR_FLOCK, _("cannot get %s lock on database"), 
+			 _("shared"));
 		return 1;
 	    } 
 	}
@@ -217,7 +219,7 @@ int openDatabase(char * prefix, char * dbpath, rpmdb *rpmdbp, int mode,
 	}
     }
 
-    *rpmdbp = malloc(sizeof(struct rpmdb));
+    *rpmdbp = malloc(sizeof(struct rpmdb_s));
     **rpmdbp = db;
 
     if (justcheck) {
@@ -254,7 +256,27 @@ Header rpmdbGetRecord(rpmdb db, unsigned int offset) {
 }
 
 int rpmdbFindByFile(rpmdb db, char * filespec, dbiIndexSet * matches) {
-    return dbiSearchIndex(db->fileIndex, filespec, matches);
+    char * fs;
+    char * src, * dst;
+
+    /* we try and canonicalize the filespec a bit before doing the search */
+
+    fs = alloca(strlen(filespec) + 1);
+    for (src = filespec, dst = fs; *src; ) { 
+	switch (*src) {
+	  case '/':
+	    if ((dst == fs) || (*(dst - 1) != '/'))
+		*dst++ = *src;
+	    src++;
+	    break;
+	  default:
+	    *dst++ = *src++;
+	}
+    }
+    *dst-- = '\0';
+    if (*dst == '/') *dst = '\0';
+
+    return dbiSearchIndex(db->fileIndex, fs, matches);
 }
 
 int rpmdbFindByProvides(rpmdb db, char * filespec, dbiIndexSet * matches) {
@@ -286,7 +308,7 @@ static void removeIndexEntry(dbiIndex * dbi, char * key, dbiIndexRecord rec,
     switch (rc) {
       case 0:
 	if (dbiRemoveIndexRecord(&matches, rec) && !tolerant) {
-	    rpmError(RPMERR_DBCORRUPT, "package %s not listed in %s",
+	    rpmError(RPMERR_DBCORRUPT, _("package %s not listed in %s"),
 		  key, idxName);
 	} else {
 	    dbiUpdateIndex(dbi, key, &matches);
@@ -297,7 +319,8 @@ static void removeIndexEntry(dbiIndex * dbi, char * key, dbiIndexRecord rec,
 	break;
       case 1:
 	if (!tolerant) 
-	    rpmError(RPMERR_DBCORRUPT, "package %s not found in %s", key, idxName);
+	    rpmError(RPMERR_DBCORRUPT, _("package %s not found in %s"), 
+			key, idxName);
 	break;
       case 2:
 	break;   /* error message already generated from dbindex.c */
@@ -319,7 +342,7 @@ int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant) {
 
     h = rpmdbGetRecord(db, offset);
     if (!h) {
-	rpmError(RPMERR_DBCORRUPT, "cannot read header at %d for uninstall",
+	rpmError(RPMERR_DBCORRUPT, _("cannot read header at %d for uninstall"),
 	      offset);
 	return 1;
     }
@@ -327,7 +350,7 @@ int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant) {
     blockSignals();
 
     if (!headerGetEntry(h, RPMTAG_NAME, &type, (void **) &name, &count)) {
-	rpmError(RPMERR_DBCORRUPT, "package has no name");
+	rpmError(RPMERR_DBCORRUPT, _("package has no name"));
     } else {
 	rpmMessage(RPMMESS_DEBUG, "removing name index\n");
 	removeIndexEntry(db->nameIndex, name, rec, tolerant, "name index");
@@ -353,11 +376,15 @@ int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant) {
 
     if (headerGetEntry(h, RPMTAG_REQUIRENAME, &type, (void **) &requiredbyList, 
 	 &count)) {
+	/* There could be dups in requiredByLIst, and the list is sorted.
+	   Rather then sort the list, be tolerant of missing entries
+	   as they should just indicate duplicated requirements. */
+
 	for (i = 0; i < count; i++) {
 	    rpmMessage(RPMMESS_DEBUG, "removing requiredby index for %s\n", 
 		    requiredbyList[i]);
 	    removeIndexEntry(db->requiredbyIndex, requiredbyList[i], rec, 
-			     tolerant, "requiredby index");
+			     1, "requiredby index");
 	}
 	free(requiredbyList);
     }
@@ -462,7 +489,7 @@ int rpmdbAdd(rpmdb db, Header dbentry) {
 
     dboffset = faAlloc(db->pkgs, headerSizeof(dbentry, HEADER_MAGIC_NO));
     if (!dboffset) {
-	rpmError(RPMERR_DBCORRUPT, "cannot allocate space for database");
+	rpmError(RPMERR_DBCORRUPT, _("cannot allocate space for database"));
 	unblockSignals();
 	if (providesCount) free(providesList);
 	if (requiredbyCount) free(requiredbyList);
@@ -520,7 +547,7 @@ int rpmdbUpdateRecord(rpmdb db, int offset, Header newHeader) {
 
     oldHeader = rpmdbGetRecord(db, offset);
     if (!oldHeader) {
-	rpmError(RPMERR_DBCORRUPT, "cannot read header at %d for update",
+	rpmError(RPMERR_DBCORRUPT, _("cannot read header at %d for update"),
 		offset);
 	return 1;
     }
@@ -591,6 +618,9 @@ void rpmdbRemoveDatabase(char * rootdir, char * dbpath) {
 
     sprintf(filename, "%s/%s/providesindex.rpm", rootdir, dbpath);
     unlink(filename);
+
+    sprintf(filename, "%s/%s/conflictsindex.rpm", rootdir, dbpath);
+    unlink(filename);
 }
 
 int rpmdbMoveDatabase(char * rootdir, char * olddbpath, char * newdbpath) {
@@ -643,5 +673,9 @@ int rpmdbMoveDatabase(char * rootdir, char * olddbpath, char * newdbpath) {
     sprintf(nfilename, "%s/%s/providesindex.rpm", rootdir, newdbpath);
     if (rename(ofilename, nfilename)) rc = 1;
 
+    sprintf(ofilename, "%s/%s/conflictsindex.rpm", rootdir, olddbpath);
+    sprintf(nfilename, "%s/%s/conflictsindex.rpm", rootdir, newdbpath);
+    if (rename(ofilename, nfilename)) rc = 1;
+
     return rc;
 }
