@@ -1,4 +1,7 @@
-#ifdef HAVE_MACHINE_TYPES_H
+#include "config.h"
+#include "miscfn.h"
+
+#if HAVE_MACHINE_TYPES_H
 # include <machine/types.h>
 #endif
 
@@ -32,7 +35,9 @@ int writeLead(int fd, struct rpmlead *lead)
     l.osnum = htons(l.osnum);
     l.signature_type = htons(l.signature_type);
 	
-    write(fd, &l, sizeof(l));
+    if (write(fd, &l, sizeof(l)) < 0) {
+	return 1;
+    }
 
     return 0;
 }
