@@ -19,6 +19,9 @@ int rpmReadPackageHeader(int fd, Header * hdr, int * isSource, int * major,
 extern const struct headerTagTableEntry rpmTagTable[];
 extern const int rpmTagTableSize;
 
+/* this chains to headerDefaultFormats[] */
+extern const struct headerSprintfExtension rpmHeaderFormats[];
+
 /* these tags are for both the database and packages */
 /* none of these can be 0 !!                         */
 
@@ -63,8 +66,8 @@ extern const int rpmTagTableSize;
 #define RPMTAG_ROOT                     1038
 #define RPMTAG_FILEUSERNAME             1039
 #define RPMTAG_FILEGROUPNAME            1040
-#define RPMTAG_EXCLUDE                  1041 /* not used - internal */
-#define RPMTAG_EXCLUSIVE                1042 /* not used - internal */
+#define RPMTAG_EXCLUDE                  1041 /* not used - depricated */
+#define RPMTAG_EXCLUSIVE                1042 /* not used - depricated */
 #define RPMTAG_ICON                     1043
 #define RPMTAG_SOURCERPM                1044
 #define RPMTAG_FILEVERIFYFLAGS          1045
@@ -96,6 +99,21 @@ extern const int rpmTagTableSize;
 #define RPMTAG_CHANGELOGTIME            1080
 #define RPMTAG_CHANGELOGNAME            1081
 #define RPMTAG_CHANGELOGTEXT            1082
+#define RPMTAG_BROKENMD5                1083 /* internal */
+#define RPMTAG_PREREQ                   1084 /* internal */
+#define RPMTAG_PREINPROG                1085
+#define RPMTAG_POSTINPROG               1086
+#define RPMTAG_PREUNPROG                1087
+#define RPMTAG_POSTUNPROG               1088
+#define RPMTAG_BUILDARCHS               1089
+#define RPMTAG_OBSOLETES                1090
+#define RPMTAG_VERIFYSCRIPTPROG         1091
+#define RPMTAG_TRIGGERSCRIPTPROGS       1092
+#define RPMTAG_DOCDIR                   1093 /* internal */
+#define RPMTAG_COOKIE                   1094
+#define RPMTAG_FILEDEVICES              1095
+#define RPMTAG_FILEINODES               1096
+#define RPMTAG_FILELANGS                1097
 
 #define RPMTAG_EXTERNAL_TAG		1000000
 
@@ -107,8 +125,12 @@ extern const int rpmTagTableSize;
 /* these can be ORed together */
 #define RPMFILE_CONFIG			(1 << 0)
 #define RPMFILE_DOC			(1 << 1)
-#define RPMFILE_SPECFILE                (1 << 2)
-
+#define RPMFILE_DONOTUSE                (1 << 2)
+#define RPMFILE_MISSINGOK               (1 << 3)
+#define RPMFILE_NOREPLACE               (1 << 4)
+#define RPMFILE_SPECFILE                (1 << 5)
+#define RPMFILE_GHOST                   (1 << 6)
+    
 #define RPMINSTALL_REPLACEPKG           (1 << 0)
 #define RPMINSTALL_REPLACEFILES         (1 << 1)
 #define RPMINSTALL_TEST                 (1 << 2)
@@ -118,9 +140,13 @@ extern const int rpmTagTableSize;
 #define RPMINSTALL_NOSCRIPTS            (1 << 6)
 #define RPMINSTALL_NOARCH               (1 << 7)
 #define RPMINSTALL_NOOS                 (1 << 8)
+#define RPMINSTALL_ALLFILES             (1 << 9)
+#define RPMINSTALL_JUSTDB               (1 << 10)
+#define RPMINSTALL_KEEPOBSOLETE         (1 << 11)
 
 #define RPMUNINSTALL_TEST               (1 << 0)
 #define RPMUNINSTALL_NOSCRIPTS          (1 << 1)
+#define RPMUNINSTALL_JUSTDB             (1 << 2)
 
 #define RPMVERIFY_NONE             0
 #define RPMVERIFY_MD5              (1 << 0)
@@ -140,10 +166,13 @@ extern const int rpmTagTableSize;
 #define RPMSENSE_EQUAL           (1 << 3)
 #define RPMSENSE_PROVIDES        (1 << 4) /* only used internally by builds */
 #define RPMSENSE_CONFLICTS       (1 << 5) /* only used internally by builds */
-#define RPMSENSE_SENSEMASK       15       /* Mask to get senses */
+#define RPMSENSE_PREREQ          (1 << 6)
+#define RPMSENSE_OBSOLETES       (1 << 7) /* only used internally by builds */
+#define RPMSENSE_SENSEMASK       15       /* Mask to get senses, ie serial, */
+                                          /* less, greater, equal.          */
 
-#define RPMSENSE_TRIGGER_ON              (1 << 16)
-#define RPMSENSE_TRIGGER_OFF             (1 << 17)
+#define RPMSENSE_TRIGGER_IN      (1 << 16)
+#define RPMSENSE_TRIGGER_UN      (1 << 17)
 
 /* Stuff for maintaining "variables" like SOURCEDIR, BUILDDIR, etc */
 
@@ -178,31 +207,45 @@ extern const int rpmTagTableSize;
 #define RPMVAR_PACKAGER                 28
 #define RPMVAR_FTPPROXY                 29
 #define RPMVAR_TMPPATH                  30
-#define RPMVAR_CPIOBIN                  31
+/* #define RPMVAR_CPIOBIN                  31 -- No longer used */
 #define RPMVAR_FTPPORT			32
 #define RPMVAR_NETSHAREDPATH		33
 #define RPMVAR_DEFAULTDOCDIR		34
 #define RPMVAR_FIXPERMS			35
-#define RPMVAR_LASTVAR	                36 /* IMPORTANT to keep right! */
+#define RPMVAR_GZIPBIN     		36
+#define RPMVAR_RPMFILENAME     		37
+#define RPMVAR_PROVIDES     		38
 
-char *rpmGetVar(int var);
+#define RPMVAR_NUM			39     /* number of RPMVAR entries */
+
+char * rpmGetVar(int var);
 int rpmGetBooleanVar(int var);
 void rpmSetVar(int var, char *val);
 
 /** rpmrc.c **/
 
+#define RPM_MACHTABLE_INSTARCH		0
+#define RPM_MACHTABLE_INSTOS		1
+#define RPM_MACHTABLE_BUILDARCH		2
+#define RPM_MACHTABLE_BUILDOS  		3
+#define RPM_MACHTABLE_COUNT		4	/* number of arch/os tables */
+
+/* rpmReadConfigFiles() is for backwards compatibility only! It won't
+   work if building is true! */
 int rpmReadConfigFiles(char * file, char * arch, char * os, int building);
-int rpmGetOsNum(void);
-int rpmGetArchNum(void);
-char *rpmGetOsName(void);
-char *rpmGetArchName(void);
+int rpmReadRC(char * file);
+void rpmGetArchInfo(char ** name, int * num);
+void rpmGetOsInfo(char ** name, int * num);
+int rpmMachineScore(int type, char * name);
 int rpmShowRC(FILE *f);
-int rpmArchScore(char * arch);
-int rpmOsScore(char * os);
+void rpmSetTables(int archTable, int osTable);  /* only used by build code */
+/* if either are NULL, they are set to the default value (munged uname())
+   pushed through a translation table (if appropriate) */
+void rpmSetMachine(char * arch, char * os);
 
 /** **/
 
-typedef struct rpmdb * rpmdb;
+typedef struct rpmdb_s * rpmdb;
 
 typedef void (*rpmNotifyFunction)(const unsigned long amount,
 			       const unsigned long total);
@@ -225,8 +268,13 @@ int rpmdbFindByProvides(rpmdb db, char * provides, dbiIndexSet * matches);
 int rpmdbFindByRequiredBy(rpmdb db, char * requires, dbiIndexSet * matches);
 int rpmdbFindByConflicts(rpmdb db, char * conflicts, dbiIndexSet * matches);
 
+/* these are just convience functions */
+int rpmdbFindByLabel(rpmdb db, char * label, dbiIndexSet * matches);
+int rpmdbFindByHeader(rpmdb db, Header h, dbiIndexSet * matches);
+
 int rpmInstallSourcePackage(char * root, int fd, char ** specFile,
-			    rpmNotifyFunction notify, char * labelFormat);
+			    rpmNotifyFunction notify, char * labelFormat,
+			    char ** cookie);
 int rpmInstallPackage(char * rootdir, rpmdb db, int fd, char * prefix, 
 		      int flags, rpmNotifyFunction notify, char * labelFormat,
 		      char * netsharedPath);
@@ -251,12 +299,19 @@ struct rpmDependencyConflict {
 } ;
 
 rpmDependencies rpmdepDependencies(rpmdb db); 	       /* db may be NULL */
-void rpmdepAddPackage(rpmDependencies rpmdep, Header h);
+void rpmdepAddPackage(rpmDependencies rpmdep, Header h, void * key);
 void rpmdepAvailablePackage(rpmDependencies rpmdep, Header h, void * key);
-void rpmdepUpgradePackage(rpmDependencies rpmdep, Header h);
+void rpmdepUpgradePackage(rpmDependencies rpmdep, Header h, void * key);
 void rpmdepRemovePackage(rpmDependencies rpmdep, int dboffset);
+
+/* this checks for dependency satisfaction, but *not* ordering */
 int rpmdepCheck(rpmDependencies rpmdep, 
 		struct rpmDependencyConflict ** conflicts, int * numConflicts);
+/* Orders items, returns error on circle, finals keys[] is NULL. No dependency
+   check is done, use rpmdepCheck() for that. If dependencies are not 
+   satisfied a "best-try" ordering is returned. */
+int rpmdepOrder(rpmDependencies order, void *** keysListPtr);
+
 void rpmdepDone(rpmDependencies rpmdep);
 void rpmdepFreeConflicts(struct rpmDependencyConflict * conflicts, int
 			 numConflicts);
@@ -356,6 +411,11 @@ rpmErrorCallBackType rpmErrorSetCallback(rpmErrorCallBackType);
 #define RPMERR_NORELOCATE	-32	/* tried to relocate improper package */
 #define RPMERR_BADOS            -33     /* bad architecture or arch mismatch */
 #define RPMMESS_BACKUP          -34     /* backup made during [un]install */
+#define RPMERR_MTAB		-35	/* failed to read mount table */
+#define RPMERR_STAT		-36	/* failed to stat something */
+#define RPMERR_BADDEV		-37	/* file on device not listed in mtab */
+#define RPMMESS_ALTNAME         -38     /* file written as .rpmnew */
+#define RPMMESS_PREREQLOOP      -39     /* loop in prerequisites */
 
 /* spec.c build.c pack.c */
 #define RPMERR_UNMATCHEDIF      -107    /* unclosed %ifarch or %ifos */
@@ -370,6 +430,7 @@ rpmErrorCallBackType rpmErrorSetCallback(rpmErrorCallBackType);
 #define RPMERR_GZIP             -117
 #define RPMERR_BADSPEC          -118
 #define RPMERR_LDD              -119    /* couldn't understand ldd output */
+#define RPMERR_BADFILENAME	-120
 
 #define RPMERR_BADSIGTYPE       -200    /* Unknown signature type */
 #define RPMERR_SIGGEN           -201    /* Error generating signature */
@@ -386,10 +447,11 @@ rpmErrorCallBackType rpmErrorSetCallback(rpmErrorCallBackType);
 /**************************************************/
 
 #define RPMSIGTAG_SIZE         	        1000
-/* the md5 sum was broken on big endian machines for a while */
-#define RPMSIGTAG_LITTLEENDIANMD5	1001
+/* the md5 sum was broken *twice* on big endian machines */
+#define RPMSIGTAG_LEMD5_1		1001
 #define RPMSIGTAG_PGP          	        1002
-#define RPMSIGTAG_MD5		        1003
+#define RPMSIGTAG_LEMD5_2		1003
+#define RPMSIGTAG_MD5		        1004
 
 /**************************************************/
 /*                                                */
@@ -408,4 +470,8 @@ void rpmFreeSignature(Header h);
 int rpmVerifySignature(char *file, int_32 sigTag, void *sig, int count,
 		       char *result);
 
+int rpmGetFilesystemList(char *** listptr, int * num);
+int rpmGetFilesystemUsage(char ** filelist, int_32 * fssizes, int numFiles,
+			  uint_32 ** usagesPtr, int flags);
+
 #endif
