@@ -1,4 +1,5 @@
 #include "config.h"
+#include "miscfn.h"
 
 #if HAVE_ALLOCA_H
 # include <alloca.h>
@@ -8,12 +9,15 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
 
 #include "install.h"
+#include "intl.h"
 #include "messages.h"
 #include "md5.h"
 #include "misc.h"
@@ -21,7 +25,7 @@
 #include "rpmlib.h"
 
 static char * SCRIPT_PATH = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"
-			                 "/usr/X11R6/bin\nexport PATH\n";
+			                 "/usr/X11R6/bin";
 
 enum fileActions { REMOVE, BACKUP, KEEP };
 
@@ -123,8 +127,8 @@ static int handleSharedFiles(rpmdb db, int offset, char ** fileList,
 	    secOffset = sharedList[i].secRecOffset;
 	    sech = rpmdbGetRecord(db, secOffset);
 	    if (!sech) {
-		rpmError(RPMERR_DBCORRUPT, "cannot read header at %d for "
-		      "uninstall", offset);
+		rpmError(RPMERR_DBCORRUPT, 
+			 _("cannot read header at %d for uninstall"), offset);
 		rc = 1;
 		break;
 	    }
@@ -136,8 +140,9 @@ static int handleSharedFiles(rpmdb db, int offset, char ** fileList,
 	    headerGetEntry(sech, RPMTAG_RELEASE, &type, (void **) &release, 
 		     &secFileCount);
 
-	    rpmMessage(RPMMESS_DEBUG, "package %s-%s-%s contain shared files\n", 
-		    name, version, release);
+	    rpmMessage(RPMMESS_DEBUG, 
+			_("package %s-%s-%s contain shared files\n"), 
+		    	name, version, release);
 
 	    if (!headerGetEntry(sech, RPMTAG_FILENAMES, &type, 
 			  (void **) &secFileList, &secFileCount)) {
@@ -208,6 +213,9 @@ int rpmRemovePackage(char * prefix, rpmdb db, unsigned int offset, int flags) {
     enum { REMOVE, BACKUP, KEEP } * fileActions;
     int scriptArg;
 
+    if (flags & RPMUNINSTALL_JUSTDB)
+	flags |= RPMUNINSTALL_NOSCRIPTS;
+
     h = rpmdbGetRecord(db, offset);
     if (!h) {
 	rpmError(RPMERR_DBCORRUPT, "cannot read header at %d for uninstall",
@@ -235,17 +243,20 @@ int rpmRemovePackage(char * prefix, rpmdb db, unsigned int offset, int flags) {
 	rmmess = "removing";
     }
 
-    rpmMessage(RPMMESS_DEBUG, "running preuninstall script (if any)\n");
+    if (!(flags & RPMUNINSTALL_TEST)) {
+	rpmMessage(RPMMESS_DEBUG, "running preuninstall script (if any)\n");
 
-    if (runScript(prefix, h, RPMTAG_PREUN, scriptArg, 
-		 flags & RPMUNINSTALL_NOSCRIPTS)) {
-	headerFree(h);
-	return 1;
+	if (runScript(prefix, h, RPMTAG_PREUN, RPMTAG_PREUNPROG, scriptArg, 
+		     flags & RPMUNINSTALL_NOSCRIPTS, 0)) {
+	    headerFree(h);
+	    return 1;
+	}
     }
     
     rpmMessage(RPMMESS_DEBUG, "%s files test = %d\n", rmmess, flags & RPMUNINSTALL_TEST);
-    if (headerGetEntry(h, RPMTAG_FILENAMES, &type, (void **) &fileList, 
-	 &fileCount)) {
+    if (!(flags & RPMUNINSTALL_JUSTDB) &&
+	headerGetEntry(h, RPMTAG_FILENAMES, &type, (void **) &fileList, 
+	               &fileCount)) {
 	if (prefix[0]) {
 	    fnbuffersize = 1024;
 	    fnbuffer = alloca(fnbuffersize);
@@ -295,8 +306,11 @@ int rpmRemovePackage(char * prefix, rpmdb db, unsigned int offset, int flags) {
 	free(fileMd5List);
     }
 
-    rpmMessage(RPMMESS_DEBUG, "running postuninstall script (if any)\n");
-    runScript(prefix, h, RPMTAG_POSTUN, scriptArg, flags & RPMUNINSTALL_NOSCRIPTS);
+    if (!(flags & RPMUNINSTALL_TEST)) {
+	rpmMessage(RPMMESS_DEBUG, "running postuninstall script (if any)\n");
+	runScript(prefix, h, RPMTAG_POSTUN, RPMTAG_POSTUNPROG, scriptArg, 
+		    flags & RPMUNINSTALL_NOSCRIPTS, 0);
+    }
 
     headerFree(h);
 
@@ -307,73 +321,150 @@ int rpmRemovePackage(char * prefix, rpmdb db, unsigned int offset, int flags) {
     return 0;
 }
 
-int runScript(char * prefix, Header h, int tag, int arg, int norunScripts) {
-    int count, type;
+int runScript(char * root, Header h, int scriptTag, int progTag,
+	      int arg, int norunScripts, int err) {
     char * script;
     char * fn;
-    int fd;
+    int fd = -1;
     int isdebug = rpmIsDebug();
     int child;
     int status;
     char upgradeArg[20];
     char * installPrefix = NULL;
     char * installPrefixEnv = NULL;
-
-    sprintf(upgradeArg, "%d", arg);
+    char * program;
+    char ** programArgv;
+    char ** argv = NULL;
+    int programArgc;
+    int programType = 0;
+    int pipes[2];
+    int out = 0;
 
     if (norunScripts) return 0;
 
-    if (headerGetEntry(h, tag, &type, (void **) &script, &count)) {
-	if (headerGetEntry(h, RPMTAG_INSTALLPREFIX, &type, (void **) &installPrefix,
-	    	     &count)) {
-	    installPrefixEnv = alloca(strlen(installPrefix) + 30);
-	    strcpy(installPrefixEnv, "RPM_INSTALL_PREFIX=");
-	    strcat(installPrefixEnv, installPrefix);
-	}
+    sprintf(upgradeArg, "%d", arg);
 
-	fn = tmpnam(NULL);
-	rpmMessage(RPMMESS_DEBUG, "script found - running from file %s\n", fn);
-	fd = open(fn, O_CREAT | O_RDWR);
-	if (!isdebug) unlink(fn);
-	if (fd < 0) {
-	    rpmError(RPMERR_SCRIPT, "error creating file for (un)install script");
+    /* headerGetEntry() sets the data pointer to NULL if the entry does
+       not exist */
+    headerGetEntry(h, progTag, &programType, (void **) &programArgv,
+		   &programArgc);
+    headerGetEntry(h, scriptTag, NULL, (void **) &script, NULL);
+
+    if (!programArgv && !script)
+	return 0;
+    else if (!programArgv) {
+	programArgv = malloc(4 * sizeof(char *));
+	programArgv[0] = "/bin/sh";
+	programArgc = 1;
+    } else if (programType == RPM_STRING_TYPE) {
+	program = (char *) programArgv;
+	programArgv = malloc(4 * sizeof(char *));
+	programArgv[0] = program;
+	programArgc = 1;
+    } else {
+	argv = programArgv;
+	programArgv = malloc((programArgc + 3) * sizeof(char *));
+	memcpy(programArgv, argv, programArgc * sizeof(char *));
+    }
+
+    if (headerGetEntry(h, RPMTAG_INSTALLPREFIX, NULL, (void **) &installPrefix,
+	    	       NULL)) {
+	installPrefixEnv = alloca(strlen(installPrefix) + 30);
+	strcpy(installPrefixEnv, "RPM_INSTALL_PREFIX=");
+	strcat(installPrefixEnv, installPrefix);
+	strcat(installPrefixEnv, "\n");
+    }
+
+    rpmMessage(RPMMESS_DEBUG, "running inst helper %s\n", programArgv[0]);
+
+    if (script) {
+	if (makeTempFile(root, &fn, &fd)) {
+	    free(programArgv);
+	    if (argv)
+		free(argv);
 	    return 1;
 	}
-	write(fd, SCRIPT_PATH, strlen(SCRIPT_PATH));
+
+	if (isdebug &&
+	    (!strcmp(programArgv[0], "/bin/sh") || 
+	     !strcmp(programArgv[0], "/bin/bash")))
+	    write(fd, "set -x\n", 7);
+
+	if (installPrefixEnv)
+	    write(fd, installPrefixEnv, strlen(installPrefixEnv));
+
 	write(fd, script, strlen(script));
-	
-	/* run the script via /bin/sh - just feed the commands to the
-	   shell as stdin */
-	if (!(child = fork())) {
-	    if (installPrefixEnv) {
-		doputenv(installPrefixEnv);
-	    }
-
-	    lseek(fd, 0, SEEK_SET);
-	    close(0);
-	    dup2(fd, 0);
-	    close(fd);
-
-	    if (strcmp(prefix, "/")) {
-		rpmMessage(RPMMESS_DEBUG, "performing chroot(%s)\n", prefix);
-		chroot(prefix);
-		chdir("/");
-	    }
-
-	    if (isdebug)
-		execl("/bin/sh", "/bin/sh", "-xs", upgradeArg, NULL);
-	    else
-		execl("/bin/sh", "/bin/sh", "-s", upgradeArg, NULL);
-	    _exit(-1);
-	}
 	close(fd);
-	waitpid(child, &status, 0);
 
-	if (!WIFEXITED(status) || WEXITSTATUS(status)) {
-	    rpmError(RPMERR_SCRIPT, "execution of script failed");
-	    return 1;
+	programArgv[programArgc++] = fn + strlen(root);
+	programArgv[programArgc++] = upgradeArg;
+    }
+
+    programArgv[programArgc] = NULL;
+
+    if (err) {
+	if (rpmIsVerbose()) {
+	    out = err;
+	} else {
+	    out = open("/dev/null", O_WRONLY);
+	    if (out < 0) {
+		out = err;
+	    }
 	}
     }
+    
+    if (!(child = fork())) {
+	/* make stdin inaccessible */
+	pipe(pipes);
+	close(pipes[1]);
+	dup2(pipes[0], 0);
+	close(pipes[0]);
+
+	if (err) {
+	    if (err != 2) dup2(err, 2);
+	    if (out != 1) dup2(out, 1);
+	    /* make sure we don't close stdin/stderr/stdout by mistake! */
+	    /* if (err > 2) close (err);
+	    if (out > 2 && out != err) close (out); */
+	}
+
+	doputenv(SCRIPT_PATH);
+	if (installPrefixEnv) {
+	    doputenv(installPrefixEnv);
+	}
+
+	if (strcmp(root, "/")) {
+	    rpmMessage(RPMMESS_DEBUG, "performing chroot(%s)\n", root);
+	    chroot(root);
+	}
+
+	chdir("/");
+
+	execv(programArgv[0], programArgv);
+ 	_exit(-1);
+    }
+
+    waitpid(child, &status, 0);
+
+    if (err) {
+	if (out > 2) close(out);
+	if (err > 2) close(err);
+	if (!rpmIsVerbose()) close(out);
+    }
+    
+    free(programArgv);
+    if (argv)
+	free(argv);
+    
+    if (script) {
+	if (!isdebug) unlink(fn);
+	free(fn);
+    }
+
+    if (!WIFEXITED(status) || WEXITSTATUS(status)) {
+	rpmError(RPMERR_SCRIPT, _("execution of script failed"));
+	return 1;
+    }
 
     return 0;
 }
@@ -400,13 +491,15 @@ static int removeFile(char * file, char state, unsigned int flags, char * md5,
 		rc = mdfile(file, currentMd5);
 
 	    if (mdfile(file, currentMd5)) 
-		rpmMessage(RPMMESS_DEBUG, "    failed - assuming file removed\n");
+		rpmMessage(RPMMESS_DEBUG, 
+				"    failed - assuming file removed\n");
 	    else {
 		if (strcmp(currentMd5, md5)) {
 		    rpmMessage(RPMMESS_DEBUG, "    file changed - will save\n");
 		    action = BACKUP;
 		} else {
-		    rpmMessage(RPMMESS_DEBUG, "    file unchanged - will remove\n");
+		    rpmMessage(RPMMESS_DEBUG, 
+				"    file unchanged - will remove\n");
 		}
 	    }
 	}
@@ -424,7 +517,7 @@ static int removeFile(char * file, char state, unsigned int flags, char * md5,
 		strcpy(newfile, file);
 		strcat(newfile, ".rpmsave");
 		if (rename(file, newfile)) {
-		    rpmError(RPMERR_RENAME, "rename of %s to %s failed: %s",
+		    rpmError(RPMERR_RENAME, _("rename of %s to %s failed: %s"),
 				file, newfile, strerror(errno));
 		    rc = 1;
 		}
@@ -437,10 +530,11 @@ static int removeFile(char * file, char state, unsigned int flags, char * md5,
 		if (!test) {
 		    if (rmdir(file)) {
 			if (errno == ENOTEMPTY)
-			    rpmError(RPMERR_RMDIR, "cannot remove %s - directory "
-				  "not empty", file);
+			    rpmError(RPMERR_RMDIR, 
+				_("cannot remove %s - directory not empty"), 
+				file);
 			else
-			    rpmError(RPMERR_RMDIR, "rmdir of %s failed: %s",
+			    rpmError(RPMERR_RMDIR, _("rmdir of %s failed: %s"),
 					file, strerror(errno));
 			rc = 1;
 		    }
@@ -448,8 +542,11 @@ static int removeFile(char * file, char state, unsigned int flags, char * md5,
 	    } else {
 		if (!test) {
 		    if (unlink(file)) {
-			rpmError(RPMERR_UNLINK, "removal of %s failed: %s",
-				    file, strerror(errno));
+			if (errno != ENOENT || !(flags & RPMFILE_MISSINGOK)) {
+			    rpmError(RPMERR_UNLINK, 
+				      _("removal of %s failed: %s"),
+					file, strerror(errno));
+			}
 			rc = 1;
 		    }
 		}
