@@ -1,4 +1,5 @@
 #include "config.h"
+#include "miscfn.h"
 
 #if HAVE_ALLOCA_H
 # include <alloca.h>
@@ -8,6 +9,8 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -16,12 +19,10 @@
 #include "misc.h"
 #include "messages.h"
 #include "rpmlib.h"
+#include "install.h"
 
 #define S_ISDEV(m) (S_ISBLK((m)) || S_ISCHR((m)))
 
-static char * SCRIPT_PATH = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"
-			                 "/usr/X11R6/bin\nexport PATH\n";
-
 int rpmVerifyFile(char * prefix, Header h, int filenum, int * result, 
 		  int omitMask) {
     char ** fileList, ** md5List, ** linktoList;
@@ -30,6 +31,8 @@ int rpmVerifyFile(char * prefix, Header h, int filenum, int * result,
     unsigned short * modeList, * rdevList;
     char * fileStatesList;
     char * filespec;
+    char * name;
+    gid_t gid;
     int type, count, rc;
     struct stat sb;
     unsigned char md5sum[40];
@@ -37,12 +40,29 @@ int rpmVerifyFile(char * prefix, Header h, int filenum, int * result,
     char linkto[1024];
     int size;
     char ** unameList, ** gnameList;
-    int useBrokenMd5;
+    int_32 useBrokenMd5;
+#if WORDS_BIGENDIAN
+    int_32 * brokenPtr;
+#endif
 
-    if (headerGetEntry(h, RPMTAG_RPMVERSION, NULL, NULL, NULL))
-	useBrokenMd5 = 0;
-    else
-	useBrokenMd5 = 1;
+#if WORDS_BIGENDIAN
+    if (!headerGetEntry(h, RPMTAG_BROKENMD5, NULL, (void **) &brokenPtr, NULL)) {
+	char * rpmVersion;
+
+	if (headerGetEntry(h, RPMTAG_RPMVERSION, NULL, (void **) &rpmVersion, 
+				NULL)) {
+	    useBrokenMd5 = ((rpmvercmp(rpmVersion, "2.3.3") >= 0) &&
+			    (rpmvercmp(rpmVersion, "2.3.8") <= 0));
+	} else {
+	    useBrokenMd5 = 1;
+	}
+	headerAddEntry(h, RPMTAG_BROKENMD5, RPM_INT32_TYPE, &useBrokenMd5, 1);
+    } else {
+	useBrokenMd5 = *brokenPtr;
+    }
+#else
+    useBrokenMd5 = 0;
+#endif
 
     headerGetEntry(h, RPMTAG_FILEMODES, &type, (void **) &modeList, &count);
 
@@ -76,8 +96,12 @@ int rpmVerifyFile(char * prefix, Header h, int filenum, int * result,
     if (S_ISDIR(sb.st_mode))
 	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
 			RPMVERIFY_LINKTO);
-    else if (S_ISLNK(sb.st_mode))
+    else if (S_ISLNK(sb.st_mode)) {
 	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME);
+        #if CHOWN_FOLLOWS_SYMLINK
+	    flags &= ~(RPMVERIFY_USER | RPMVERIFY_GROUP);
+        #endif
+    }
     else if (S_ISFIFO(sb.st_mode))
 	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
 			RPMVERIFY_LINKTO);
@@ -149,7 +173,8 @@ int rpmVerifyFile(char * prefix, Header h, int filenum, int * result,
     if (flags & RPMVERIFY_USER) {
 	if (headerGetEntry(h, RPMTAG_FILEUSERNAME, NULL, (void **) &unameList, 
 			   NULL)) {
-	    if (strcmp(unameList[filenum], uidToUname(sb.st_uid)))
+	    name = uidToUname(sb.st_uid);
+	    if (!name || strcmp(unameList[filenum], name))
 		*result |= RPMVERIFY_USER;
 	    free(unameList);
 	} else if (headerGetEntry(h, RPMTAG_FILEUIDS, NULL, (void **) &uidList, 
@@ -166,7 +191,8 @@ int rpmVerifyFile(char * prefix, Header h, int filenum, int * result,
     if (flags & RPMVERIFY_GROUP) {
 	if (headerGetEntry(h, RPMTAG_FILEGROUPNAME, NULL, (void **) &gnameList, 
 			NULL)) {
-	    if (strcmp(gnameList[filenum], gidToGname(sb.st_gid)))
+	    rc =  gnameToGid(gnameList[filenum],&gid);
+	    if (rc || (gid != sb.st_gid))
 		*result |= RPMVERIFY_GROUP;
 	    free(gnameList);
 	} else if (headerGetEntry(h, RPMTAG_FILEGIDS, NULL, (void **) &gidList, 
@@ -184,77 +210,6 @@ int rpmVerifyFile(char * prefix, Header h, int filenum, int * result,
 }
 
 int rpmVerifyScript(char * root, Header h, int err) {
-    int out, fd;
-    char * script;
-    char * fn;
-    char * tmpdir = rpmGetVar(RPMVAR_TMPPATH);
-    int status;
-
-    if (!headerGetEntry(h, RPMTAG_VERIFYSCRIPT, NULL, (void **) &script, 
-			NULL)) {
-	return 0;
-    }
-
-    if (rpmIsVerbose()) {
-	out = err;
-    } else {
-	out = open("/dev/null", O_APPEND);
-	if (out < 0) {
-	    out = err;
-	}
-    }
-
-    fn = alloca(strlen(tmpdir) + 20);
-    sprintf(fn, "%s/rpm-%d.vscript", tmpdir, (int) getpid());
-
-    rpmMessage(RPMMESS_DEBUG, "verify script found - "
-		"running from file %s\n", fn);
-
-    fd = open(fn, O_CREAT | O_RDWR);
-    unlink(fn);
-    if (fd < 0) {
-	rpmError(RPMERR_SCRIPT, "error creating file for verify script");
-	return 1;
-    }
-    write(fd, SCRIPT_PATH, strlen(SCRIPT_PATH));
-    write(fd, script, strlen(script));
-    lseek(fd, 0, SEEK_SET);
-
-    if (!fork()) {
-	dup2(fd, 0);
-	close(fd);
-
-	if (err != 2) dup2(err, 2);
-        if (out != 1) dup2(out, 1);
-
-	/* make sure we don't close stdin/stderr/stdout by mistake! */
-	if (err > 2) close (err);
-	if (out > 2 && out != err) close (out);
-
-	if (strcmp(root, "/")) {
-	    rpmMessage(RPMMESS_DEBUG, "performing chroot(%s)\n", root);
-	    chroot(root);
-	    chdir("/");
-	}
-
-	if (rpmIsDebug())
-	    execl("/bin/sh", "/bin/sh", "-x", "-s", NULL);
-	else
-	    execl("/bin/sh", "/bin/sh", "-s", NULL);
-	exit(-1);
-    }
-
-    if (out > 2) close(out);
-    if (err > 2) close(err);
-    close(fd);
-    if (!rpmIsVerbose()) close(out);
-
-    wait(&status);
-
-    if (!WIFEXITED(status) || WEXITSTATUS(status)) {
-	rpmError(RPMERR_SCRIPT, "execution of verify script failed");
-	return 1;
-    }
-
-    return 0;
+    return runScript(root, h, RPMTAG_VERIFYSCRIPT, RPMTAG_VERIFYSCRIPTPROG,
+		     0, 0, err);
 }
