@@ -21,10 +21,10 @@ int main(int argc, char **argv)
     }
 
     readLead(fd, &lead);
-    readSignature(fd, NULL, lead.signature_type);
-    hd = readHeader(fd, (lead.major >= 3) ?
-		    HEADER_MAGIC : NO_HEADER_MAGIC);
-    writeHeader(1, hd, HEADER_MAGIC);
+    rpmReadSignature(fd, NULL, lead.signature_type);
+    hd = headerRead(fd, (lead.major >= 3) ?
+		    HEADER_MAGIC_YES : HEADER_MAGIC_NO);
+    headerWrite(1, hd, HEADER_MAGIC_YES);
     
     return 0;
 }
