@@ -1,4 +1,4 @@
-#include "config.h"
+#include "miscfn.h"
 
 #include <fcntl.h>
 #include <pwd.h>
@@ -13,6 +13,7 @@
 
 #include "ftp.h"
 #include "messages.h"
+#include "miscfn.h"
 #include "rpmlib.h"
 #include "url.h"
 
